unit sio;

{$mode objfpc}{$H+}

interface

uses
  //CPort,
  Classes, SysUtils;

type
  PSioHeader = ^TSioHeader;
  TSioHeader = record
    stb      : byte;
    ComGroup : byte;
    ComID    : byte;
    Version  : byte;
    Sender   : byte;
    Receiver : byte;
    Length   : word;
  end;

  TSioTelegramm = object
    cnt     : byte;
    //ComPort : TComPort;
    data    : array of byte;
    constructor Create({port:TComPort;} sender,receiver:byte);
    destructor Done;
    procedure SetLength(len:word);
    procedure SetString(str:string);
    procedure CRC;
    function X16_Checksum:word;
    procedure send(group,id:byte);
  end;

var
  SioTelegram1 : TSioTelegramm;

implementation

const
  TestString = '123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
  XOR_CHECKSUM_START = $29DB; // Start value for the rotating XOR Checksum

var
  s_header    : TSioHeader;
  s_telegramm : TSioTelegramm;


  function TSioTelegramm.X16_Checksum:word;
  var
    crc1 : longword; // only 16 bits used, but 32 bit auto variables produce faster assembler code
    i    : integer;
  begin
    crc1 := XOR_CHECKSUM_START;    // init value

    for i:=0 to PSioHeader(data)^.Length+SizeOf(TSioHeader)-1 do begin
      crc1 := crc1 xor data[i]; // first XOR...
      if (crc1 and $0001) > 0 then begin
        crc1 := crc1 shr 1;   // ...then rotate, bit 0 was "1"
        crc1 := crc1 or $8000;
      end else begin
        crc1 := crc1 shr 1;   // ...then rotate, bit 0 war "0"
      end;
    end;
    result := crc1;
  end;

  function X16_Checksum_TEST:word;
  const
    TestString : string = '123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
  var
    crc1 : longword; // only 16 bits used, but 32 bit auto variables produce faster assembler code
    i    : integer;
  begin
    crc1 := XOR_CHECKSUM_START;    // init value

    for i:=1 to Length(TestString) do begin
      crc1 := crc1 xor longword(TestString[i]); // first XOR...
      if (crc1 and $0001) > 0 then begin
        crc1 := crc1 shr 1;   // ...then rotate, bit 0 was "1"
        crc1 := crc1 or $8000;
      end else begin
        crc1 := crc1 shr 1;   // ...then rotate, bit 0 war "0"
      end;
    end;
    result := crc1;
  end;


  procedure X16_ChecksumNextByte(var crc1:word; byte1:byte);
  begin
    crc1 := crc1 xor byte1; // first XOR...
    if (crc1 and $0001) > 0 then begin
      crc1 := crc1 shr 1;   // ...then rotate, bit 0 was "1"
      crc1 := crc1 or $8000;
    end else begin
      crc1 := crc1 shr 1;   // ...then rotate, bit 0 war "0"
    end;
  end;

constructor TSioTelegramm.Create({port:TComPort;} sender,receiver:byte);
var
  header : TSioHeader absolute data;
begin
  //ComPort := port;
  SetLength(0);
  cnt := 1;
  header.stb      := $41;
  header.ComGroup := 0;
  header.ComID    := 0;
  header.Version  := cnt;
  header.Sender   := sender;
  Receiver := receiver;
  header.Length   := 0;
end;

destructor TSioTelegramm.Done;
begin
end;

procedure TSioTelegramm.send(group,id:byte);
var
  header : TSioHeader absolute data;
  data2 : array of word absolute data;
begin
  header.ComGroup := group;
  header.ComGroup := id;
  data2[SizeOf(TSioHeader)+header.Length] := X16_Checksum;
  //ComPort.Write(data[0], header.Length+SizeOf(TSioHeader)+2{CRC});
  inc(cnt);
end;

procedure TSioTelegramm.SetLength(len:word);
begin
  System.SetLength(data, len+SizeOf(TSioHeader)+2+len);
  PSioHeader(data)^.Length := len;
end;

procedure TSioTelegramm.SetString(str:string);
begin
  SetLength(Length(str));
  Move(str[1], data[sizeof(TSioHeader)], Length(str));
end;

procedure TSioTelegramm.CRC;
begin
end;

begin
  SioTelegram1.Create({nil,} $CF, $F1);
  SioTelegram1.send($50, $49);
end.

